#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "ppm.h"

PPM	*ppm_alloc(int r, int c)
{
	PPM *img = NULL;

  // allocate struct
  img = (PPM *)malloc(sizeof(PPM));

  if(!img) {
    fprintf(stderr,"WARNING: could not allocate memory for img\n");
  }

  // initialize struct contents
  img->ptype = 6;
  img->h = r;
  img->w = c;
  img->max = 255;
  img->bpp = 3;
  img->name = NULL;
  img->rpix = (float *)malloc((r*c)*sizeof(float));
  img->gpix = (float *)malloc((r*c)*sizeof(float));
  img->bpix = (float *)malloc((r*c)*sizeof(float));

  if(!img->rpix || !img->gpix || !img->bpix) {
    fprintf(stderr,"WARNING: could not allocate memory for img data\n");
  }

  return img;
}

void     ppm_free(PPM **img)
{
  // free internals, anything that got allocated
  if((*img)->name)
    free((*img)->name);

  if((*img)->rpix)
    free((*img)->rpix);

  if((*img)->gpix)
    free((*img)->gpix);

  if((*img)->bpix)
    free((*img)->bpix);

  free((*img));
  (*img) = NULL;
}

PPM     *ppm_read(const char *file)
{
	int		i,j;
	int		rows,cols,maxc;
	char		c;
	PPM 		*img = NULL;
	FILE		*in = NULL;
	unsigned char	byte;

  // open file
  in = fopen(file,"r");
  if(!in) {
    fprintf(stderr,"Error in opening %s\n", file);
    return img;
  }

  // read input
  if( ((c=fgetc(in))=='P') && ((c=fgetc(in))=='6') && ((c=fgetc(in))=='\n')) {

    // ppm images can have # comments after first line
    if((c=fgetc(in))=='#')
      do {
        c=fgetc(in);
      } while(c!='\n');
    else
      ungetc(c,in);

    // read width, height
    if(fscanf(in,"%u %u",&cols,&rows) < 2)
      fprintf(stderr,"Error getting width, height from %s\n", file);

    // read maxc
    if( (fscanf(in,"%u\n",&maxc) < 1) || maxc != 255 )
      fprintf(stderr,"Error getting maxc from %s\n", file);

    img = ppm_alloc(rows,cols);

    if(img) {
      // read in data
      for(i=0;i<rows;i++) {
        for(j=0;j<cols;j++) {

          if( (fread(&byte,sizeof(unsigned char),1,in) < 1) || feof(in) )
            fprintf(stderr,"Error getting r from %s\n", file);
          img->rpix[i*img->w + j] = (float)byte/(float)maxc;

          if( (fread(&byte,sizeof(unsigned char),1,in) < 1) || feof(in) )
            fprintf(stderr,"Error getting g from %s\n", file);
          img->gpix[i*img->w + j] = (float)byte/(float)maxc;

          if( (fread(&byte,sizeof(unsigned char),1,in) < 1) || feof(in) )
            fprintf(stderr,"Error getting b from %s\n", file);
          img->bpix[i*img->w + j] = (float)byte/(float)maxc;

        }
      }
    }

  } else {
    fprintf(stderr,"Unknown character in %s\n", file);
  }

  return img;
}

int      ppm_write(PPM *img, const char *file)
{
	int		i,j;
	FILE		*out;
	unsigned char	byte;

  // open output file for writing
  out = fopen(file,"w");
  if(!out) {
    printf("Error in opening %s\n", file);
    return 1;
  }

  // write output
  fprintf(out,"P%d\n%d %d\n%d\n",img->ptype,img->h,img->w,img->max);
  for(i=0;i<img->h;i++) {
    for(j=0;j<img->w;j++) {
      byte = (unsigned char)(img->rpix[i*img->w + j] * (float)img->max);
      fwrite(&byte,sizeof(unsigned char),1,out);
      byte = (unsigned char)(img->gpix[i*img->w + j] * (float)img->max);
      fwrite(&byte,sizeof(unsigned char),1,out);
      byte = (unsigned char)(img->bpix[i*img->w + j] * (float)img->max);
      fwrite(&byte,sizeof(unsigned char),1,out);
    }
  }
  fflush(out);

  fclose(out);

  return 0;
}

void     ppm_white(PPM *img)
{
	int i,j;

  for(i=0; i<img->h; i++) {
    for(j=0; j<img->w; j++) {
      img->rpix[i*img->w + j] = 1.0;
      img->gpix[i*img->w + j] = 1.0;
      img->bpix[i*img->w + j] = 1.0;
    }
  }
}
