#include <fstream>
#include <iostream>
#include <string>
#include <cstring>

#include "pgm.h"

std::istream& operator>>(std::istream& s, pgm_t& rhs)
{
	char		c,byte;

  // read 'P' if there
  c = s.get();
  if(s.eof()) {
    std::cerr << "Warning: EOF / read error reading magic number" << std::endl;
    return s;
  }
  if(c != 'P') {
    std::cerr << "Warning: Unknown image file type, not PNM" << std::endl;
    return s;
  }
  // get image type, eating whitespace at EOL
  s >> rhs.ptype >> std::ws;
  if(s.eof()) {
    std::cerr << "Warning: EOF / read error reading magic number" << std::endl;
    return s;
  }

  // determine bytes per pixel
  switch(rhs.ptype) {
    case 5: rhs.bpp = 1; break;
    case 6: rhs.bpp = 3; break;
    case 7: rhs.bpp = 4; break;
  }

  // eat comment lines
  while((c = s.peek()) == '#') do { s.get(c); } while(c != '\n');

  // get w,h, eating ws at EOL
  s >> rhs.w >> rhs.h >> std::ws;

  // get max (color), eating ws at EOL
  s >> rhs.max >> std::ws;

  // allocate and read the byte
  rhs.gpix = new float [rhs.w * rhs.h];

  // read uchar byte
////byte = new char [rhs.bpp * rhs.w * rhs.h];
////s.read(byte,rhs.bpp*rhs.w*rhs.h);
//  byte = new char [rhs.w * rhs.h];
//  s.read(byte,rhs.w*rhs.h);

  if(rhs.gpix) {
    for(int i=0;i<rhs.h;i++) {
      for(int j=0;j<rhs.w;j++) {
        s.read(&byte,1);
        rhs.gpix[i*rhs.w + j] = (float)byte/(float)rhs.max;
      }
    }
  }

//delete byte;

  return s;
}

std::ostream& operator<<(std::ostream& s, const pgm_t& rhs)
{
//	char	c, *byte = new char [rhs.w * rhs.h];
	char	c, byte;

  s << 'P' << rhs.ptype << std::endl;
  s << "# Stored by pgm_t class." << std::endl;
  s << rhs.w << " " << rhs.h << std::endl;
  s << rhs.max << std::endl;

  for(int i=0;i<rhs.h;i++) {
    for(int j=0;j<rhs.w;j++) {
      byte = (char)(rhs.gpix[i*rhs.w + j]*(float)rhs.max);
      s.write(&byte,1);
    }
  }

  return s;
}

const pgm_t& pgm_t::operator=(const pgm_t& rhs)
{
  if(this != &rhs) {	// standard alias test
    // copy all elements of rhs into this
    ptype = rhs.ptype;
    max = rhs.max;
    w = rhs.w;  h = rhs.h;
    bpp = rhs.bpp;
    if(gpix) delete gpix;
    gpix = new float [w * h];
    memcpy(gpix,rhs.gpix,w*h*sizeof(float));
  }
  return *this;
}

pgm_t::pgm_t(const pgm_t& rhs)
{
  // copy constructor creates the pgm_t object given a constant reference to
  // another pgm_t object (result is a copy of the other object)
  // copy all elements of rhs into this except for name
  ptype = rhs.ptype;
  max = rhs.max;
  w = rhs.w;  h = rhs.h;
  bpp = rhs.bpp;
//gpix = new char [rhs.bpp * rhs.w * rhs.h];
  gpix = new float [rhs.w * rhs.h];

//memcpy(gpix,rhs.gpix,rhs.bpp*rhs.w*rhs.h);
  memcpy(gpix,rhs.gpix,rhs.w*rhs.h*sizeof(float));
}
