#include <fstream>
#include <iostream>
#include <string>
#include <cstring>

#include "pgm.h"
#include "ppm.h"

std::istream& operator>>(std::istream& s, ppm_t& rhs)
{
	char	c, *rdata, *gdata, *bdata;

  // read 'P' if there
  c = s.get();
  if(s.eof()) {
    std::cerr << "Warning: EOF / read error reading magic number" << std::endl;
    return s;
  }
  if(c != 'P') {
    std::cerr << "Warning: Unknown image file type, not PNM" << std::endl;
    return s;
  }
  // get image type, eating whitespace at EOL
  s >> rhs.ptype >> std::ws;
  if(s.eof()) {
    std::cerr << "Warning: EOF / read error reading magic number" << std::endl;
    return s;
  }

  // determine bytes per pixel
  switch(rhs.ptype) {
    case 5: rhs.bpp = 1; break;
    case 6: rhs.bpp = 3; break;
    case 7: rhs.bpp = 4; break;
  }

  // eat comment lines
  while((c = s.peek()) == '#') do { s.get(c); } while(c != '\n');

  // get w,h, eating ws at EOL
  s >> rhs.w >> rhs.h >> std::ws;

  // get max (color), eating ws at EOL
  s >> rhs.max >> std::ws;

  // allocate and read the data
  rhs.rpix = new float [rhs.w * rhs.h];
  rhs.gpix = new float [rhs.w * rhs.h];
  rhs.bpix = new float [rhs.w * rhs.h];

  // read uchar data
//data = new char [rhs.bpp * rhs.w * rhs.h];
//s.read(data,rhs.bpp*rhs.w*rhs.h);
//data = new char [rhs.w * rhs.h];
//s.read(data,rhs.w*rhs.h);
  rdata = new char [rhs.w * rhs.h];
  gdata = new char [rhs.w * rhs.h];
  bdata = new char [rhs.w * rhs.h];
  for(int i=0;i<rhs.h;i++) {
    for(int j=0;j<rhs.w;j++) {
      s.read(&rdata[i*rhs.w + j],1);
      s.read(&gdata[i*rhs.w + j],1);
      s.read(&bdata[i*rhs.w + j],1);
    }
  }

  if(rhs.gpix) {
    for(int i=0;i<rhs.h;i++) {
      for(int j=0;j<rhs.w;j++) {
        rhs.rpix[i*rhs.w + j] = (float)rdata[i*rhs.w + j]/(float)rhs.max;
        rhs.gpix[i*rhs.w + j] = (float)gdata[i*rhs.w + j]/(float)rhs.max;
        rhs.bpix[i*rhs.w + j] = (float)bdata[i*rhs.w + j]/(float)rhs.max;
      }
    }
  }

  delete [] rdata;
  delete [] gdata;
  delete [] bdata;

  return s;
}

std::ostream& operator<<(std::ostream& s, const ppm_t& rhs)
{
	char	*rdata = new char [rhs.w * rhs.h];
	char	*gdata = new char [rhs.w * rhs.h];
	char	*bdata = new char [rhs.w * rhs.h];

  for(int i=0;i<rhs.h;i++) {
    for(int j=0;j<rhs.w;j++) {
      rdata[i*rhs.w + j] = (char)(rhs.rpix[i*rhs.w + j]*(float)rhs.max);
      gdata[i*rhs.w + j] = (char)(rhs.gpix[i*rhs.w + j]*(float)rhs.max);
      bdata[i*rhs.w + j] = (char)(rhs.bpix[i*rhs.w + j]*(float)rhs.max);
    }
  }

  s << 'P' << rhs.ptype << std::endl;
  s << "# Stored by ppm_t class." << std::endl;
  s << rhs.w << " " << rhs.h << std::endl << rhs.max << std::endl;

//s.write(rhs.gpix,rhs.bpp*rhs.w*rhs.h);
  for(int i=0;i<rhs.h;i++) {
    for(int j=0;j<rhs.w;j++) {
      s.write(&rdata[i*rhs.w + j],1);
      s.write(&gdata[i*rhs.w + j],1);
      s.write(&bdata[i*rhs.w + j],1);
    }
  }

  delete [] rdata;
  delete [] gdata;
  delete [] bdata;

  return s;
}

const ppm_t& ppm_t::operator=(const ppm_t& rhs)
{
  if(this != &rhs) {	// standard alias test
    // copy all elements of rhs into this
    ptype = rhs.ptype;
    max = rhs.max;
    w = rhs.w;  h = rhs.h;
    bpp = rhs.bpp;

    if(rpix) delete [] rpix;
    rpix = new float [w * h];
    memcpy(rpix,rhs.rpix,w*h*sizeof(float));

    if(gpix) delete [] gpix;
    gpix = new float [w * h];
    memcpy(gpix,rhs.gpix,w*h*sizeof(float));

    if(bpix) delete [] bpix;
    bpix = new float [w * h];
    memcpy(bpix,rhs.bpix,w*h*sizeof(float));
  }
  return *this;
}

ppm_t::ppm_t(const ppm_t& rhs)
{
  // copy constructor creates the pgm_t object given a constant reference to
  // another pgm_t object (result is a copy of the other object)
  // copy all elements of rhs into this except for name
  ptype = rhs.ptype;
  max = rhs.max;
  w = rhs.w;  h = rhs.h;
  bpp = rhs.bpp;
  rpix = new float [rhs.w * rhs.h];
  gpix = new float [rhs.w * rhs.h];
  bpix = new float [rhs.w * rhs.h];

  memcpy(rpix,rhs.rpix,rhs.w*rhs.h*sizeof(float));
  memcpy(gpix,rhs.gpix,rhs.w*rhs.h*sizeof(float));
  memcpy(bpix,rhs.bpix,rhs.w*rhs.h*sizeof(float));
}
